use exe::pe::VecPE;
use exe::types::ResourceDirectory;
use image::io::Reader as ImageReader;

use std::fs::File;
use std::io::{Cursor, Read};
use std::path::Path;

pub fn extract_icon(file: &Path) -> Result<Option<Vec<u8>>, Box<dyn std::error::Error>> {
    let mut buffer = [0u8; 1024 * 512]; // 512kb should be enough to extract icons
    let mut file = File::open(file)?;
    file.read_exact(&mut buffer)?;

    let image = VecPE::from_data(exe::PEType::Disk, buffer);
    let import_directory = ResourceDirectory::parse(&image)?;

    let mut icon = None;

    if let Some(ic) = import_directory.icon_groups(&image)?.iter().next() {
        let ico_data = ic.1.to_icon_buffer(&image)?;
        let img =
            ImageReader::with_format(Cursor::new(&ico_data), image::ImageFormat::Ico).decode()?;
        let img = img.resize_to_fill(20, 22, image::imageops::FilterType::Lanczos3);

        let mut buffer = Cursor::new(vec![]);
        img.write_to(&mut buffer, image::ImageFormat::Png)?;

        icon = Some(buffer.into_inner());
    }

    Ok(icon)
}
